@@ -33,8 +33,7 @@ sub build {
     croak "cannot build engine without type and name "
       unless $name and $type;
 
-    my $class_name = ucfirst($type);
-    my $namespace  = "Dancer::${class_name}";
+    my $class_name = $class->_engine_class($type);
 
     $config ||= {};
     $config->{engines} ||= {};
@@ -42,7 +41,7 @@ sub build {
 
     # trying to load the engine
     my $engine_class =
-      Dancer::ModuleLoader->class_from_setting($namespace => $name);
+      Dancer::ModuleLoader->class_from_setting($class_name => $name);
 
     croak "unknown $type engine '$name', "
       . "perhaps you need to install $engine_class?"
@@ -56,6 +55,18 @@ sub build {
     );
 }
 
+sub _engine_class {
+    my ($class, $type) = @_;
+    $type = ucfirst($type);
+    return "Dancer::${type}";
+}
+
+sub engine {
+    my ($class, $type) = @_;
+    return $class->_engine_class($type)->engine();
+}
+
+
 1;
 
 __END__
