@@ -4,6 +4,7 @@ use strict;
 use warnings;
 use Carp 'croak';
 
+use File::stat;
 use HTTP::Headers;
 
 use Dancer::Logger;
@@ -45,6 +46,7 @@ sub handle_request {
     my ($self, $request) = @_;
     my $ip_addr = $request->remote_address || '-';
 
+    Dancer::SharedData->reset_response();
     Dancer::SharedData->reset_timer;
     Dancer::Logger::core("request: "
           . $request->method . " "
@@ -66,9 +68,8 @@ sub handle_request {
         Dancer::App->reload_apps;
     }
 
-    my $response;
     eval {
-             $response = Dancer::Renderer->render_file
+        Dancer::Renderer->render_file
           || Dancer::Renderer->render_action
           || Dancer::Renderer->render_error(404);
     };
@@ -76,14 +77,13 @@ sub handle_request {
         Dancer::Logger::core(
             'request to ' . $request->path_info . " crashed: $@");
 
-        my $error = Dancer::Error->new(
+        Dancer::Error->new(
             code    => 500,
             title   => "Runtime Error",
             message => $@
-        );
-        $response = $error->render;
+        )->render();
     }
-    return $self->render_response($response);
+    return $self->render_response();
 }
 
 sub psgi_app {
@@ -112,25 +112,47 @@ sub init_request_headers {
 # render a PSGI-formated response from a response built by
 # handle_request()
 sub render_response {
-    my ($self, $response) = @_;
+    my $self     = shift;
+    my $response = Dancer::SharedData->response();
 
-    my $content = $response->{content};
-    unless (ref($content) eq 'GLOB') {
+    my $content = $response->content;
 
+    unless ( ref($content) eq 'GLOB' ) {
         my $charset = setting('charset');
         my $ctype   = $response->header('Content-Type');
 
-        if ($charset && $ctype && _is_text($ctype)) {
-            $content = Encode::encode($charset, $content);
-            $response->header('Content-Type' => "$ctype; charset=$charset")
+        if ( $charset && $ctype && _is_text($ctype) ) {
+            $content = Encode::encode( $charset, $content );
+            $response->header( 'Content-Type' => "$ctype; charset=$charset" )
               if $ctype !~ /$charset/;
         }
+        $response->header( 'Content-Length' => length($content) )
+          if !defined $response->header('Content-Length');
         $content = [$content];
     }
+    else {
+        if ( !defined $response->header('Content-Length') ) {
+            my $stat = stat $content;
+            $response->header( 'Content-Length' => $stat->size );
+        }
+    }
 
-    Dancer::Logger::core("response: " . $response->{status});
-    Dancer::SharedData->reset_all();
-    return [$response->{status}, $response->headers_to_array, $content];
+    # drop content if request is HEAD
+    $content = ['']
+      if ( defined Dancer::SharedData->request
+        && Dancer::SharedData->request->is_head() );
+
+    # drop content AND content_length if reponse is 1xx or (2|3)04
+    if ($response->status =~ (/^[23]04$/ )) {
+        $content = [''];
+        $response->header('Content-Length' => 0);
+    }
+    
+    Dancer::Logger::core("response: " . $response->status);
+
+    my $status  = $response->status();
+    my $headers = $response->headers_to_array();
+    return [ $status, $headers, $content ];
 }
 
 sub _is_text {
